"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useIonHeaderParallax = void 0;
const React = __importStar(require("react"));
function useIonHeaderParallax({ image, defaultImage = image, titleColor = '#AAA', expandedColor = '#313131', maximumHeight = 300, showBarButtons = false, }) {
    const [ticking, setTicking] = React.useState(false);
    const [loading, setLoading] = React.useState(true);
    const ref = React.useRef(null);
    React.useEffect(() => {
        setLoading(true);
        const timeout = setTimeout(() => {
            initElements();
            setLoading(false);
        }, 300);
        return () => {
            clearTimeout(timeout);
        };
    }, [image, titleColor, expandedColor, maximumHeight, ref.current]);
    const initElements = () => {
        var _a;
        // ion-header
        if (ref && ref.current) {
            const header = ref.current;
            const parentElement = header.parentElement;
            if (!parentElement)
                throw new Error('No IonPage parent element');
            // ion-toolbar
            const toolbar = header.querySelector('ion-toolbar');
            if (!toolbar)
                throw new Error('Parallax requires an <ion-toolbar> element on the page to work');
            // ion-toolbar background
            const toolbarShadowRoot = toolbar.shadowRoot;
            if (!toolbarShadowRoot)
                throw new Error('No shadow');
            const toolbarBackground = toolbarShadowRoot.querySelector('.toolbar-background');
            // ion-buttons
            const barButtons = header.querySelector('ion-buttons');
            // ion-content
            const ionContent = parentElement.querySelector('ion-content');
            if (!ionContent)
                throw new Error('Parallax requires an <ion-content> element on the page to work.');
            const scrollContent = (_a = ionContent.shadowRoot) === null || _a === void 0 ? void 0 : _a.querySelector('.inner-scroll');
            if (!scrollContent) {
                return;
                //throw new Error('Parallax directive requires an <ion-content> element on the page to work.')
            }
            // create image overlay
            const imageOverlay = document.createElement('div');
            imageOverlay.classList.add('image-overlay');
            const colorOverlay = document.createElement('div');
            colorOverlay.classList.add('color-overlay');
            colorOverlay.appendChild(imageOverlay);
            header.appendChild(colorOverlay);
            if (barButtons) {
                imageOverlay.appendChild(barButtons);
            }
            /***  initStyles ***/
            // still in init use JS DOM
            setTicking(false);
            maximumHeight = parseFloat(maximumHeight.toString());
            let headerMinHeight = toolbar.offsetHeight;
            // fetch styles
            const originalToolbarBgColor = window.getComputedStyle(toolbarBackground, null).backgroundColor;
            if (!originalToolbarBgColor) {
                throw new Error('Error: toolbarBackround is null.');
            }
            // header and title
            header.style.position = 'relative';
            // color overlay
            colorOverlay.style.backgroundColor = originalToolbarBgColor;
            colorOverlay.style.height = `${maximumHeight}px`;
            colorOverlay.style.position = 'absolute';
            colorOverlay.style.top = `${-headerMinHeight * 0}px`;
            colorOverlay.style.left = '0';
            colorOverlay.style.width = '100%';
            colorOverlay.style.zIndex = '10';
            colorOverlay.style.pointerEvents = 'none';
            // image overlay
            imageOverlay.style.backgroundColor = expandedColor;
            imageOverlay.style.backgroundImage = `url(${image || defaultImage})`;
            imageOverlay.style.height = '100%';
            imageOverlay.style.width = '100%';
            imageOverlay.style.pointerEvents = 'none';
            imageOverlay.style.backgroundSize = 'cover';
            imageOverlay.style.backgroundPosition = 'center';
            // .toolbar-background
            toolbarBackground.style.backgroundColor = originalToolbarBgColor;
            // .bar-buttons
            if (barButtons) {
                barButtons.style.pointerEvents = 'all';
                Array.from(barButtons.children).forEach((btn) => {
                    const htmlBtn = btn;
                    htmlBtn.style.color = titleColor;
                });
            }
            // .scroll-content
            if (scrollContent) {
                scrollContent.style.paddingTop = `${imageOverlay.clientHeight - toolbar.clientHeight}px`;
                scrollContent.addEventListener('scroll', (_e) => {
                    if (!ticking) {
                        window.requestAnimationFrame(() => {
                            // to do
                            const scrollTop = scrollContent.scrollTop;
                            // Parallax total progress
                            headerMinHeight = toolbar.offsetHeight;
                            let progress = (maximumHeight - scrollTop - headerMinHeight) / (maximumHeight - headerMinHeight);
                            progress = Math.max(progress, 0);
                            let targetHeight = maximumHeight - scrollTop;
                            targetHeight = Math.max(targetHeight, headerMinHeight);
                            // .toolbar-background: change color
                            imageOverlay.style.height = `${targetHeight}px`;
                            imageOverlay.style.opacity = `${progress}`;
                            colorOverlay.style.height = `${targetHeight}px`;
                            colorOverlay.style.opacity = targetHeight > headerMinHeight ? '1' : '0';
                            /* TODO: add support for dark mode for toolbar*/
                            toolbarBackground.style.backgroundColor =
                                targetHeight > headerMinHeight ? 'transparent' : originalToolbarBgColor;
                            // .bar-buttons
                            if (barButtons && showBarButtons) {
                                if (targetHeight > headerMinHeight) {
                                    imageOverlay.append(barButtons);
                                    Array.from(barButtons.children).forEach((btn) => {
                                        const htmlBtn = btn;
                                        if (htmlBtn) {
                                            htmlBtn.style.color = titleColor;
                                            // allow custom styles TODO
                                        }
                                    });
                                }
                                else {
                                    toolbar.append(barButtons);
                                    Array.from(barButtons.children).forEach((btn) => {
                                        const htmlBtn = btn;
                                        if (htmlBtn) {
                                            htmlBtn.style.color = 'unset';
                                            // allow custom styles TODO
                                        }
                                    });
                                }
                            }
                        });
                    }
                    setTicking(true);
                });
            }
        }
    };
    return {
        ref,
        loading,
    };
}
exports.useIonHeaderParallax = useIonHeaderParallax;
